######### Input box ############
#### Sell= 50 , Buy= 2.5 , SL= 70%

entry_time=time(9,45,0,0)   ##### entry time in hr, min, sec for strategy
num_strikes=71   ##### max no of strikes to consider: i/p odd number like 25,35,etc --> changeable
expiry= date(2023,9,13)      #### expiry selection date(yyyy,mm,dd)--> changeable
Buy_prm_closest= 2.5 ## BUY Closest premium i/p
Delay_buy=1  ## Seconds delay in sell
Sell_prm_closest= 50 ## SELL Closest premium i/p
Stop_loss=1.7 ##for 70% stoploss, Stop_loss = 1.7
lots=4 ## qty=lots*perlotqty, perlotqty is 40 for FINNIFTY, 15 for BANKNIFTY, 75 for MIDCPNIFTY, 50 for NIFTY

##FOr 1 LOT, Margin is approx 1.1L for BANKNIFTY PE and CE SELL-ONLY, 50k for BANKNIFTY PE and CE BUY-SELL
##FOr 1 LOT, Margin is approx 1.1L for FINNIFTY PE and CE SELL-ONLY, 50k for BANKNIFTY PE and CE BUY-SELL

underlying_print="BANKNIFTY"  ### "FINNIFTY", "MIDCPNIFTY", "BANKNIFTY", "NIFTY"
buy_sell_order="BS"   #### "SO" SELL-ONLY, "BS" BUY-SELL 
same_day_sqoff="E"  #### "E" for ENABLED, "D" for DISABLED
squareoff_time=time(15,15,0,0)


strike_step_dict={"BANKNIFTY": 100, "FINNIFTY":50, "MIDCPNIFTY": 50, "NIFTY": 50} ###100 for BN, 50 for FN, MIDCPNIFTY, NIFTY
strike_step=strike_step_dict[underlying_print]
underlying_dict={"FINNIFTY":999920041, "MIDCPNIFTY": 999920043, "BANKNIFTY": 999920005, "NIFTY": 999920000} 
scripcode_underlying=underlying_dict[underlying_print]
lotsize={"FINNIFTY":40, "MIDCPNIFTY": 75, "BANKNIFTY": 15, "NIFTY": 50}
lotsize_underlying=lotsize[underlying_print]
qty=int(lots*lotsize_underlying)
buy_sell_print={"SO":"SELL-ONLY", "BS": "BUY AND SELL"}

print("Input time =",datetime.now(pytz.timezone('Asia/Kolkata')))
print("::::::::::::Inputting successful:::::::::::::::::::")



pd.set_option('display.max_rows', 1000) ## to make max rows visible
pd.set_option('display.max_columns', 1000) ## to make max columns visible




############# Import library modules #############
import pandas as pd
from datetime import datetime, date, time
import pytz # for timezone

###########Scripmaster Input########
def scripmaster_download_func():
    #Scripmaster = pd.read_csv('scripmaster-csv-format.csv')
    Scripmaster = pd.read_csv('https://images.5paisa.com/website/scripmaster-csv-format.csv')
    print("Last scripmaster download on =", datetime.now(pytz.timezone('Asia/Kolkata')))
    return Scripmaster


from py5paisa.order import Order, OrderType, Exchange
import pandas as pd
import time as tym
from datetime import datetime, date, time
import pytz # for timezone

def sqoff_place(client, iDict, oDict):
    
    CE_Scripcode_SELL=oDict['CE_Scripcode_SELL']
    CE_Scripcode_BUY=oDict['CE_Scripcode_BUY']
    PE_Scripcode_SELL=oDict['PE_Scripcode_SELL']
    PE_Scripcode_BUY=oDict['PE_Scripcode_BUY']
    
    qty=iDict['qty']
    buy_sell_order=iDict['buy_sell_order']
    same_day_sqoff=iDict['same_day_sqoff']
    Delay_buy=iDict['Delay_buy']
    Stop_loss=iDict['Stop_loss']
    squareoff_time=iDict['squareoff_time']


    ############## SAME DAY SL and AUTOSQUAREOFF CODE###########
    if same_day_sqoff=="E":
        print("Waiting for square-off at", squareoff_time)
        while True:
            tym.sleep(5)
            if squareoff_time<datetime.now(pytz.timezone('Asia/Kolkata')).time():
                print("Autosquareoff started at=",datetime.now(pytz.timezone('Asia/Kolkata')).time())
                client.squareoff_all()
                tym.sleep(5)
                client.squareoff_all()
                a="square off all completed"
                print(a)
                break
    else:
        a="auto_squareoff disabled"
        print(a)
    return a


import pandas as pd
import numpy as np
from datetime import datetime, date, time
import time as tym
import pytz # for timezone
from py5paisa.order import Order, OrderType, Exchange
from IPython.display import clear_output, display


def df_strikeprice_func(client, iDict, Scripmaster):

    entry_time=iDict['entry_time']
    num_strikes=iDict['num_strikes']
    expiry=iDict['expiry'] 
    Buy_prm_closest=iDict['Buy_prm_closest']
    Delay_buy=iDict['Delay_buy'] 
    Sell_prm_closest=iDict['Sell_prm_closest'] 
    qty=iDict['qty']
    Stop_loss=iDict['Stop_loss']
    strike_step=iDict['strike_step']
    underlying_print=iDict['underlying_print']
    Scripcode_underlying=iDict['Scripcode_underlying'] 
    
    start = tym.time() ## time counter start
    ######### Codes for fetching underlying live value into df_strikeprice ##########
    today=datetime.now(pytz.timezone('Asia/Kolkata')).date()
    code_start=datetime.now(pytz.timezone('Asia/Kolkata')).time()
    print("New Code start time=",code_start)
    #will work on Saturday too with client.historical data fetching last market day data for today
    #print(today)
    underlying= client.historical_data('N','C',Scripcode_underlying,'1m',today,today)
    ## historical_data(<Exchange>,<Exchange Type>,<Scrip Code>,<Time Frame>,<From Data>,<To Date>)
    ## Exchange: N- NSE, B- BSE, M- Mcx
    ## Exchange Type: exchange_segment C- Cash, D- Derivative, U-Currency
    ## Scripcode: Refer Scripmaster
    ## Time Frame: 1m, 5m, 10m, 15m, 30m, 60m, 1d
    ## From Date: Start date
    ## To  Date: End Date

    ######### Codes for strikes generation for Underlying into df_strikeprice ##########

    underlying_ref= round((underlying["Close"][len(underlying)-1])/strike_step,0)*strike_step

    df_strikeprice=pd.DataFrame(index=range(num_strikes))
    df_strikeprice["Strike_Selection"]=np.nan  ## filling  strike price column with nan values in new column
    df_strikeprice["Scripcode_CE"]=np.nan ## filling CE scripcode column with nan values in new column
    df_strikeprice["Scripcode_PE"]=np.nan ## filling  PE scripcode columnwith nan values in new column
    df_strikeprice["CE_LTP"]=np.nan ## filling  CE LTP columnwidth nan values in new column
    df_strikeprice["PE_LTP"]=np.nan ## filling  PE LTP columnwidth nan values in new column

    for i in range(num_strikes):   ## finding all the strikes referring ATM calculated as above
        df_strikeprice["Strike_Selection"][i]=underlying_ref - strike_step*((num_strikes-1)/2-i)


    ######### Codes for Scripcode selection for scripcodes of above strikes and output into df_strikeprice ##########

    #Scripmaster = pd.read_csv('scripmaster-csv-format.csv')  ## to read csv files

    #Convert datetime like "01-01-2023 14:00:00" in datetime column to date
    Scripmaster['Expiry'] = pd.to_datetime(Scripmaster['Expiry']).dt.date
    #Alternate method Scripmaster['Expiry']=pd.to_datetime(Scripmaster['Expiry'], format='%Y-%m-%d').dt.date


    Scripmaster_CE=Scripmaster[(Scripmaster.ISIN == underlying_print)  & (Scripmaster.Expiry == expiry) & (Scripmaster.CpType=="CE")]  ## Setting filter in columns
    Scripmaster_PE=Scripmaster[(Scripmaster.ISIN == underlying_print) & (Scripmaster.Expiry == expiry) & (Scripmaster.CpType=="PE")] ## Setting filter in columns
    #Scripmaster_CE and Scripmaster_PE are temporary dataframe tables for filtering current expiry PE scripcode values from Scripmaster

    Scripmaster_CE.reset_index(inplace=True) ## Resetting the index and after slicing dataframe
    Scripmaster_PE.reset_index(inplace=True) ## Resetting the index and after slicing dataframe

    for i in range(len(df_strikeprice.index)):  ## looking for strikes from data extr from CSV and saving respective scripcodes
        for j in range(len(Scripmaster_CE.index)):
            if Scripmaster_CE['StrikeRate'][j]==df_strikeprice['Strike_Selection'][i]:
                df_strikeprice['Scripcode_CE'][i]=Scripmaster_CE['Scripcode'][j]
        for k in range(len(Scripmaster_PE.index)):
            if Scripmaster_PE['StrikeRate'][k]==df_strikeprice['Strike_Selection'][i]:
                df_strikeprice['Scripcode_PE'][i]=Scripmaster_PE['Scripcode'][k]
                
    mid1 = tym.time()
    ###########Code for fetching CE and PE LTPs of strike prices in each row from 5PAISA live data##########

    temp_dataframe=pd.DataFrame() ##temporary dataframe for receiving historical price table
    temp_empty=pd.DataFrame() ##empty dataframe only used for making temp_dataframe empty

    for i in range(len(df_strikeprice.index)):    # if len is 2, range(len) is 0,1,2. for i in range(len) starts from i=0 and doesnot execute for i=len
        if np.isnan(df_strikeprice['Scripcode_CE'][i])==False:
            temp_dataframe=client.historical_data('N','d',int(df_strikeprice['Scripcode_CE'][i]),'1m',today,today) ##from 5paisa
            if (temp_dataframe is not None)==0:
                temp_dataframe=temp_empty
            elif temp_dataframe.empty==False:
                df_strikeprice.loc[i,'CE_LTP']=temp_dataframe['Close'].iloc[-1]
                temp_dataframe=temp_empty  ## clear temp_dataframe using empty dataframe temp_empty
            else:
                temp_dataframe=temp_empty
        if np.isnan(df_strikeprice['CE_LTP'][i]):
            df_strikeprice.loc[i,'CE_LTP']=10000 #replacing NaN with large no like 10000


        if np.isnan(df_strikeprice['Scripcode_PE'][i])==False:
            temp_dataframe=client.historical_data('N','d',int(df_strikeprice['Scripcode_PE'][i]),'1m',today,today) ##from 5paisa
            if (temp_dataframe.empty is not None)==0:
                temp_dataframe=temp_empty
            elif temp_dataframe.empty==False:
                df_strikeprice.loc[i,'PE_LTP']=temp_dataframe['Close'].iloc[-1]
                temp_dataframe=temp_empty  ## clear temp_dataframe using empty dataframe temp_empty
            else:
                temp_dataframe=temp_empty
        if np.isnan(df_strikeprice['PE_LTP'][i]):
            df_strikeprice.loc[i,'PE_LTP']=10000 #replacing NaN with large no like 10000

    mid2 = tym.time()

    ########Code for finding Scripcode of CE nearest to 110 and 10 premiums#######
    lst = df_strikeprice['CE_LTP'].to_numpy()

    CE_indx_SELL= (np.abs(lst - Sell_prm_closest)).argmin()
    CE_indx_BUY= (np.abs(lst - Buy_prm_closest)).argmin()

    CE_Scripcode_SELL=df_strikeprice.loc[CE_indx_SELL,'Scripcode_CE']
    CE_Scripcode_SELL = getattr(CE_Scripcode_SELL, "tolist", lambda: CE_Scripcode_SELL)() ##convert numpy int32 to native int to avoid 5paisa json serializable error
    CE_Scripcode_BUY=df_strikeprice.loc[CE_indx_BUY,'Scripcode_CE']
    CE_Scripcode_BUY = getattr(CE_Scripcode_BUY, "tolist", lambda: CE_Scripcode_BUY)() ##convert numpy int32 to native int to avoid 5paisa json serializable error

    ########Code for finding Scripcode of PE nearest to 110 and 10 premiums#######
    lst = df_strikeprice['PE_LTP'].to_numpy()
    PE_indx_SELL= (np.abs(lst - Sell_prm_closest)).argmin()
    PE_indx_BUY= (np.abs(lst - Buy_prm_closest)).argmin()

    PE_Scripcode_SELL=df_strikeprice.loc[PE_indx_SELL,'Scripcode_PE']
    PE_Scripcode_SELL = getattr(PE_Scripcode_SELL, "tolist", lambda: PE_Scripcode_SELL)() ##convert numpy int32 to native int to avoid 5paisa json serializable error
    PE_Scripcode_BUY=df_strikeprice.loc[PE_indx_BUY,'Scripcode_PE']
    PE_Scripcode_BUY = getattr(PE_Scripcode_BUY, "tolist", lambda: PE_Scripcode_BUY)() ##convert numpy int32 to native int to avoid 5paisa json serializable error 
    
    oDict={'CE_Scripcode_SELL':CE_Scripcode_SELL,
            'CE_Scripcode_BUY':CE_Scripcode_BUY,
            'PE_Scripcode_SELL':PE_Scripcode_SELL,
            'PE_Scripcode_BUY':PE_Scripcode_BUY}
    
    code_completion=tym.time()
    ##########Print strike selection, scripcode and LTPs#######################
    clear_output()
    print("Expiry date=",expiry)
    print("Code start time=",code_start)
    print("time till fetching data from 5paisa=", mid1-start)
    print("Time for fetching LTPs from 5paisa/above looptime=", mid2-mid1)
    print("Time after loop=",code_completion-mid2)
    print("Code Completion time=",code_completion-start)

    print("\n",underlying_print," LTP= ",round(underlying["Close"][len(underlying)-1],0))
    print("Strike CE=",df_strikeprice.loc[CE_indx_SELL,'Strike_Selection'],
        "  Price=",df_strikeprice.loc[CE_indx_SELL,'CE_LTP'],
        "   Scripcode=",df_strikeprice.loc[CE_indx_SELL,'Scripcode_CE'])
    print("Strike PE=",df_strikeprice.loc[PE_indx_SELL,'Strike_Selection'],
        "  Price=",df_strikeprice.loc[PE_indx_SELL,'PE_LTP'],
        "   Scripcode=",df_strikeprice.loc[PE_indx_SELL,'Scripcode_PE'])
    print("Strike CE=",df_strikeprice.loc[CE_indx_BUY,'Strike_Selection'],
        "  Price=",df_strikeprice.loc[CE_indx_BUY,'CE_LTP'],
        "   Scripcode=",df_strikeprice.loc[CE_indx_BUY,'Scripcode_CE'])
    print("Strike PE=",df_strikeprice.loc[PE_indx_BUY,'Strike_Selection'],
        "  Price=",df_strikeprice.loc[PE_indx_BUY,'PE_LTP'],
        "   Scripcode=",df_strikeprice.loc[PE_indx_BUY,'Scripcode_PE'])
    return [df_strikeprice,oDict]


import pandas as pd
from datetime import datetime, date, time
import time as tym
import pytz # for timezone
from py5paisa.order import Order, OrderType, Exchange

def create_logbook(client):
    ###########For creating log##############
    order_book=client.order_book()
    order_book=pd.json_normalize(order_book)
    datestring=datetime.now(pytz.timezone('Asia/Kolkata')).strftime("%Y.%m.%d_%H.%M.%S")
    log=pd.DataFrame()

    for i in range(len(order_book.index)):
    # df_strikeprice.loc[i,'CE_LTP']
        log.loc[i,'ExchOrderTime1']=datetime.fromtimestamp(float(order_book['ExchOrderTime'].iloc[i][6:16]))
        log.loc[i,'Script']=order_book['ScripName'].iloc[i]
        log.loc[i,'BuySell']=order_book['BuySell'].iloc[i]
        log.loc[i,'TradedQty']=order_book['Qty'].iloc[i]
        log.loc[i,'Price']=order_book['AveragePrice'].iloc[i]

    log['Entry Date']=log['ExchOrderTime1'].dt.date
    log['Entry Time']=log['ExchOrderTime1'].dt.time

    log.loc[(log['Script'].str[22:24] == 'CE'), 'CE/PE'] = 'Call'
    log.loc[(log['Script'].str[22:24] == 'PE'), 'CE/PE'] = 'Put'
    log['strike'] = log['Script'].str[25:30]
    log.loc[(log['BuySell'] == 'B'), 'BUY/SELL'] = 'BUY'
    log.loc[(log['BuySell'] == 'S'), 'BUY/SELL'] = 'SELL'

    log['Qty']=log['TradedQty'].astype(int)
    log['price']=log['Price']
    log=log.drop(['ExchOrderTime1','Script','BuySell','TradedQty','Price'], axis=1)

    return log


from py5paisa.order import Order, OrderType, Exchange
import pandas as pd
import time as tym
from datetime import datetime, date, time
import pytz # for timezone

def sl_place(client, iDict, oDict, rec):
    
    CE_Scripcode_SELL=oDict['CE_Scripcode_SELL']
    CE_Scripcode_BUY=oDict['CE_Scripcode_BUY']
    PE_Scripcode_SELL=oDict['PE_Scripcode_SELL']
    PE_Scripcode_BUY=oDict['PE_Scripcode_BUY']
    
    qty=iDict['qty']
    buy_sell_order=iDict['buy_sell_order']
    same_day_sqoff=iDict['same_day_sqoff']
    Delay_buy=iDict['Delay_buy']
    Stop_loss=iDict['Stop_loss']
    squareoff_time=iDict['squareoff_time']
    lpp_factor=iDict['lpp_factor']


    ################ Placing Stoploss Order ######################
    print("SL order to be placed")
    df_pos=pd.json_normalize(client.positions())
    for i in range(len(df_pos.index)):
        if df_pos['NetQty'][i]<0:
            print("SL order placed")
            r=client.place_order(OrderType='B',Exchange='N',ExchangeType='D', ScripCode = int(df_pos['ScripCode'][i]),
                                Qty=int(abs(df_pos['NetQty'][i])), DisQty=0, 
                                Price=int(round(df_pos['SellAvgRate'][i]*Stop_loss*lpp_factor,0)),
                                IsIntraday=False, StopLossPrice=int(df_pos['SellAvgRate'][i]*Stop_loss), 
                                AHPlaced='N')
            if r!=None:
                rec1=pd.json_normalize(r)
                rec=pd.concat([rec,rec1])
            else:
                print("NONE response") 
            
    return rec



def pip_install_func():
    %pip install jupyterthemes 
    %pip install py5paisa
    %pip install pandas
    %pip install numpy

    %pip install py5paisa --upgrade
    
    output="RESTART KERNEL"
    return output


from py5paisa import FivePaisaClient

def fivepaisalogin():
    ############## Login into 5Paisa Account Swapna ###################
    
    cred={
        "APP_NAME":"5P50222634",
        "APP_SOURCE":"16397",
        "USER_ID":"KuLeolmFxYn",
        "PASSWORD":"buPwfrtC5RX",
        "USER_KEY":"N4ETSSr1m6Swfd4efeeoenBlYcxDOLeX",
        "ENCRYPTION_KEY":"zfsjTxU7NDy1oItYYVYr5EZlKbpv5bkj"
        }

    totp=input("input totp: ")

    client = FivePaisaClient(cred=cred)
    # New TOTP based authentication
    #client.get_totp_session('Your ClientCode','TOTP from authenticator app','Your Pin')
    client.get_totp_session(50222634,str(totp),147258)
    return client

## To generate Token
## https://dev-openapi.5paisa.com/WebVendorLogin/VLogin/Index?VendorKey=N4ETSSr1m6Swfd4efeeoenBlYcxDOLeX&ResponseURL=https://www.5paisa.com/technology/developer-apis
##Client = FivePaisaClient(cred=cred)
##Client.get_access_token('eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1bmlxdWVfbmFtZSI6IjUwNTg1ODA3Iiwicm9sZSI6Ik40RVRTU3IxbTZTd2ZkNGVmZWVvZW5CbFljeERPTGVYIiwibmJmIjoxNjg2MDIxODIxLCJleHAiOjE2ODYwMjE4NTEsImlhdCI6MTY4NjAyMTgyMX0.uU_jeye0QPopDQWhA88-dm5VehjAyZhqMt0a9ZA--n0')


import pandas as pd
import numpy as np
from datetime import datetime, date, time
import time as tym
import pytz # for timezone
from py5paisa.order import Order, OrderType, Exchange
from IPython.display import clear_output

def sl_monitor(client, iDict, rec):

    rec=rec
    Stop_loss=iDict['Stop_loss'] 

    today=datetime.now(pytz.timezone('Asia/Kolkata')).date()
    temp_dataframe=pd.DataFrame()
    clear_dataframe=pd.DataFrame()

    df_order=pd.json_normalize(client.order_book())
    df_order=df_order[(df_order.OrderStatus == "Pending")]
    df_order=df_order[['ExchOrderID','OrderStatus','SLTriggerRate','ScripCode','ScripName','Qty']]
    df_order.reset_index(inplace=True)


    df_pos=pd.json_normalize(client.positions())
    # df_pos=pd.read_pickle("df_pos3.pkl")
    df_pos=df_pos[(df_pos.NetQty<0)]
    df_pos.reset_index(inplace=True)

    prices_dataframe1=pd.DataFrame(index=range(len(df_pos.index)))
    prices_dataframe1['Close']='NaN'
    prices_dataframe1['Scripcode']='NaN'
    prices_dataframe1['ScripName']='NaN'
    prices_dataframe1['ExchOrderID']='NaN'
    prices_dataframe1['SLTriggerRate']='NaN'
    prices_dataframe1['Qty']='NaN'
    prices_dataframe1['Sell_Price']='NaN'


    for i in range(len(df_pos.index)):
        temp_dataframe=client.historical_data('N','d',int(df_pos.iloc[i]['ScripCode']),'1m',today,today)
        if (temp_dataframe is not None)==0:
            print("None historical data")
        elif temp_dataframe.empty==False:
            prices_dataframe1.loc[i,'Close']=temp_dataframe.iloc[-1]['Close']
        prices_dataframe1.loc[i,'Scripcode']=df_pos.iloc[i]['ScripCode']
        prices_dataframe1.loc[i,'ScripName']=df_pos.iloc[i]['ScripName']
        prices_dataframe1.loc[i,'Qty']=int(abs(df_pos.iloc[i]['NetQty']))
        if df_pos.loc[i,'SellAvgRate']==0:
            prices_dataframe1.loc[i,'Sell_Price']=df_pos.iloc[i]['AvgRate']
        else:
            prices_dataframe1.loc[i,'Sell_Price']=df_pos.iloc[i]['SellAvgRate']

    for i in range(len(prices_dataframe1.index)):
        for j in range(len(df_order.index)):
            if df_order.loc[j,'ScripCode']==prices_dataframe1.loc[i,'Scripcode']:
                prices_dataframe1.loc[i,'ExchOrderID']=df_order.loc[j,'ExchOrderID']
                prices_dataframe1.loc[i,'SLTriggerRate']=df_order.loc[j,'SLTriggerRate']

    tym.sleep(10)

    df_pos=pd.json_normalize(client.positions())
    # df_pos=pd.read_pickle("df_pos3.pkl")
    df_pos=df_pos[(df_pos.NetQty<0)]
    df_pos.reset_index(inplace=True)

    prices_dataframe2=pd.DataFrame(index=range(len(df_pos.index)))
    prices_dataframe2['Close']='NaN'
    prices_dataframe2['Scripcode']='NaN'
    prices_dataframe2['ScripName']='NaN'
    prices_dataframe2['ExchOrderID']='NaN'
    prices_dataframe2['SLTriggerRate']='NaN'
    prices_dataframe2['Qty']='NaN'
    prices_dataframe2['Sell_Price']='NaN'

    for i in range(len(df_pos.index)):
        temp_dataframe=client.historical_data('N','d',int(df_pos.iloc[i]['ScripCode']),'1m',today,today)
        if (temp_dataframe is not None)==0:
            print("None historical data")
        elif temp_dataframe.empty==False:
            prices_dataframe2.loc[i,'Close']=temp_dataframe.iloc[-1]['Close']
        prices_dataframe2.loc[i,'Scripcode']=df_pos.iloc[i]['ScripCode']
        prices_dataframe2.loc[i,'ScripName']=df_pos.iloc[i]['ScripName']
        prices_dataframe2.loc[i,'Qty']=int(abs(df_pos.iloc[i]['NetQty']))
        if df_pos.loc[i]['SellAvgRate']==0:
            prices_dataframe2.loc[i,'Sell_Price']=df_pos.iloc[i]['AvgRate']
        else:
            prices_dataframe2.loc[i,'Sell_Price']=df_pos.iloc[i]['SellAvgRate']

    for i in range(len(prices_dataframe2.index)):
        for j in range(len(df_order.index)):
            if df_order.loc[j,'ScripCode']==prices_dataframe2.loc[i,'Scripcode']:
                prices_dataframe2.loc[i,'ExchOrderID']=df_order.loc[j,'ExchOrderID']
                prices_dataframe2.loc[i,'SLTriggerRate']=df_order.loc[j,'SLTriggerRate']

    # prices_dataframe1.loc[1,'Close']=1000 for testing
    # prices_dataframe2.loc[1,'Close']=1000 for testing
    if(len(prices_dataframe1.index))==0:
        print("NO sell positions for SL monitoring at: ",datetime.now(pytz.timezone('Asia/Kolkata')))

    for i in range(len(prices_dataframe1.index)):
        for j in range(len(prices_dataframe2.index)):
            if prices_dataframe2.loc[j,'Scripcode']==prices_dataframe1.loc[i,'Scripcode']:
                if (prices_dataframe2.loc[j,'Close']>Stop_loss*prices_dataframe2.loc[j,'Sell_Price'])&(prices_dataframe1.loc[i,'Close']>Stop_loss*prices_dataframe1.loc[i,'Sell_Price']):
                # if (prices_dataframe2.loc[j,'Close']>-1)&(prices_dataframe1.loc[i,'Close']>-1):
                    print("At time:",datetime.now(pytz.timezone('Asia/Kolkata')),", Price beyond SL for: ",prices_dataframe2.loc[j,'ScripName'])
                    if prices_dataframe1.loc[i,'ExchOrderID']!='NaN':
                        client.cancel_order(exch_order_id=prices_dataframe1.loc[i,'ExchOrderID'])
                        print("Existing SL order cancelled")
                    print("Squaring OFF")
                    r=client.place_order(OrderType='B',Exchange='N',ExchangeType='D',ScripCode= int(prices_dataframe2.loc[j,'Scripcode']),Qty=int(prices_dataframe1.loc[i,'Qty']), DisQty=int(prices_dataframe1.loc[i,'Qty']), IsIntraday= False, IsStopLossOrder= False,
                        StopLossPrice=0, Price= 0,AHPlaced='N')   ## To place CE Buy
                    if r is not None:
                        rec1=pd.json_normalize(r)
                        rec=pd.concat([rec,rec1])
                    else:
                        print("NONE response")
                else:
                    print("At time:",datetime.now(pytz.timezone('Asia/Kolkata')),", SL monitoring OK for: ",prices_dataframe2.loc[j,'ScripName'])

return rec


from py5paisa.order import Order, OrderType, Exchange
import pandas as pd
import time as tym
from datetime import datetime, date, time
import pytz # for timezone

def order_place(client, iDict, oDict):
    
    CE_Scripcode_SELL=oDict['CE_Scripcode_SELL']
    CE_Scripcode_BUY=oDict['CE_Scripcode_BUY']
    PE_Scripcode_SELL=oDict['PE_Scripcode_SELL']
    PE_Scripcode_BUY=oDict['PE_Scripcode_BUY']
    
    qty=iDict['qty']
    buy_sell_order=iDict['buy_sell_order']
    same_day_sqoff=iDict['same_day_sqoff']
    Delay_buy=iDict['Delay_buy']
    Stop_loss=iDict['Stop_loss']
    squareoff_time=iDict['squareoff_time']
    
    
    rec=pd.DataFrame()
    
    if buy_sell_order=="BS":
        print("BUY AND SELL")
        print("CE BUY order placed")
        r=client.place_order(OrderType='B',Exchange='N',ExchangeType='D',
                    ScripCode= int(CE_Scripcode_BUY),
                    Qty=qty, DisQty=qty, IsIntraday= False, IsStopLossOrder= False,
                        StopLossPrice=0, Price= 0,AHPlaced='N')   ## To place CE Buy
        if r!=None:
            rec1=pd.json_normalize(r)
            rec=pd.concat([rec,rec1])
        else:
            print("NONE response")    

        print("PE BUY order placed")
        r=client.place_order(OrderType='B',Exchange='N',ExchangeType='D',
                    ScripCode= int(PE_Scripcode_BUY),
                    Qty=qty, DisQty=qty, IsIntraday= False, IsStopLossOrder= False,
                        StopLossPrice=0, Price= 0, AHPlaced='N')  ## To place PE Buy
        if r!=None:
            rec1=pd.json_normalize(r)
            rec=pd.concat([rec,rec1])
        else:
            print("NONE response") 
            
        tym.sleep(Delay_buy) #to add delay between buy and sell
    
    if buy_sell_order=="SO":
        print("SELL ONLY")
    
    print("CE SELL order placed")
    r=client.place_order(OrderType='S',Exchange='N',ExchangeType='D',
                ScripCode = int(CE_Scripcode_SELL),
                Qty=qty, DisQty=qty, IsIntraday= False, IsStopLossOrder= False,
                    StopLossPrice=0, Price= 0, AHPlaced='N') ## To place CE Sell
    if r!=None:
        rec1=pd.json_normalize(r)
        rec=pd.concat([rec,rec1])
    else:
        print("NONE response") 

    tym.sleep(Delay_buy) #to add delay between sell and sell
    print("PE SELL order placed")
    r=client.place_order(OrderType='S',Exchange='N',ExchangeType='D',
                ScripCode = int(PE_Scripcode_SELL),
                Qty=qty, DisQty=qty, IsIntraday= False, IsStopLossOrder= False,
                    StopLossPrice=0, Price= 0, AHPlaced='N') ## To place PE Sell
    if r!=None:
        rec1=pd.json_normalize(r)
        rec=pd.concat([rec,rec1])
    else:
        print("NONE response") 

    tym.sleep(Delay_buy)

    return rec
